import os


CONDOR_PATH = os.path.expanduser('~/.condor')
TERM_LIST_PATH = os.path.join(CONDOR_PATH, 'term_lists')
MATRIX_PATH = os.path.join(CONDOR_PATH, 'matrices')
MODEL_PATH = os.path.join(CONDOR_PATH, 'models')

for path in [CONDOR_PATH, TERM_LIST_PATH, MATRIX_PATH, MODEL_PATH]:
    if not os.path.exists(path):
        os.mkdir(path)



import pytest

from condor.normalize import PunctuationRemover
from condor.normalize import Stemmer
from condor.normalize import StopwordRemover
from condor.normalize import Lowercaser
from condor.normalize import LatexAccentRemover
from condor.normalize import CompleteNormalizer


@pytest.fixture(scope="module")
def punctuated():
    return 'hola, que hace?'


@pytest.fixture(scope="module")
def punctuation():
    return PunctuationRemover()


@pytest.fixture(scope="module")
def stemmer():
    return Stemmer()


@pytest.fixture(scope="module")
def stopwords():
    return StopwordRemover()


@pytest.fixture(scope="module")
def lowercase():
    return Lowercaser()


@pytest.fixture(scope="module")
def latex():
    return LatexAccentRemover()


@pytest.fixture(scope="module")
def complete():
    return CompleteNormalizer()


def test_punctuation_remover_removes_puchtuation(punctuation, punctuated):
    # GIVEN any state
    # WHEN the punctuation remover is used in a punctuated phrase
    result = punctuation.apply_to(punctuated)
    # THEN the result has not punctuation at all
    assert ',' not in result
    assert '?' not in result


def test_punctuation_remover_keeps_words(punctuation, punctuated):
    result = punctuation.apply_to(punctuated)
    assert 'hola que hace' == result


def test_stemmer_changes_words_by_their_stemms(stemmer):
    result = stemmer.apply_to('hola que hace')
    assert 'hol que hac' in result


def test_stopword_remover_removes_stopwords(stopwords):
    result = stopwords.apply_to('costa de concordia')
    assert 'de' not in result


def test_stopword_remover_keeps_important_words(stopwords):
    result = stopwords.apply_to('costa de concordia')
    assert 'costa concordia' == result


def test_normalizers_can_be_composed(punctuated):
    class Composed(PunctuationRemover, StopwordRemover, Stemmer):
        pass
    print(Composed.__mro__)
    normalizer = Composed()
    result = normalizer.apply_to(punctuated)
    assert 'hol hac' == result


def test_lowercase_normalizer_lowercases_tokens(lowercase):
    result = lowercase.apply_to('Title Cased Phrase')
    assert result.islower()


def test_lowercase_normalizer_keeps_all_tokens(lowercase):
    result = lowercase.apply_to('Title Cased Phrase')
    assert 'title cased phrase' == result


def test_latex_accent_remover_removes_acents_from_vowels(latex):
    assert 'áéíóú' == latex.apply_to(r"\'a\'e\'i\'o\'u")


def test_latex_accent_remover_removes_acents_from_other_formats(latex):
    assert 'áéíóú' == latex.apply_to(r"{\'a}{\'e}{\'i}{\'o}{\'u}")
    assert 'áéíóú' == latex.apply_to(r"{\'{a}}{\'{e}}{\'{i}}{\'{o}}{\'{u}}")
    assert 'áéíóú' == latex.apply_to(r"\'{a}\'{e}\'{i}\'{o}\'{u}")


def test_latex_removes_acents_from_other_characters(latex):
    assert 'ñ' == latex.apply_to(r"\~n")


def test_latex_works_for_upercased_characters(latex):
    assert 'Ñ' == latex.apply_to(r"\~N")


def test_latex_accent_remover_works_with_some_cases(latex):
    assert 'didáctica' == latex.apply_to(r"did{\'{a}}ctica")


def test_complete_normalizer_works_with_some_cases(complete):
    assert 'didact' == complete.apply_to(r"did{\'{a}}ctica")



from sqlalchemy import (
    Column,
    ForeignKey,
    Unicode,
)

from sqlalchemy.orm import relationship

from condor.models.base import (
    AuditableMixing,
    DeclarativeBase,
)


class RankingMatrix(AuditableMixing, DeclarativeBase):

    __tablename__ = 'ranking_matrix'

    term_document_matrix_eid = Column(
        Unicode(40),
        ForeignKey('term_document_matrix.eid')
    )

    kind = Column(Unicode(16), nullable=False)
    build_options = Column(Unicode(512), nullable=False)
    ranking_matrix_path = Column(Unicode(512), nullable=False)

    term_document_matrix = relationship(
        'TermDocumentMatrix',
        back_populates='ranking_matrices',
    )



from sqlalchemy import (
    Column,
    Unicode,
)

from sqlalchemy.orm import relationship

from condor.models.base import (
    AuditableMixing,
    DeclarativeBase,
)


class BibliographySet(AuditableMixing, DeclarativeBase):

    __tablename__ = 'bibliography_set'

    description = Column(Unicode, nullable=False)

    bibliographies = relationship(
        'Bibliography',
        back_populates='bibliography_set',
        cascade='all, delete-orphan',
    )

    term_document_matrices = relationship(
        'TermDocumentMatrix',
        back_populates='bibliography_set',
        cascade='all, delete-orphan',
    )



from marshmallow import Schema
from marshmallow import fields


class RecordSchema(Schema):
    uuid = fields.Str()
    title = fields.Str()
    abstract = fields.Str()
    keywords = fields.List(fields.Str)
    raw = fields.Raw()


class ModelSchema(Schema):
    words = fields.List(fields.Str)
    model_path = fields.Str()
    created_at = fields.DateTime()



'''
This script aims to build the model for the current state of a database,
taking in all the present records.
'''

import hashlib
import os
import sys

import click
import numpy

from condor.dbutil import session
from condor.builders.matrix import build_matrix
from condor.models import (
    BibliographySet,
    TermDocumentMatrix,
    RankingMatrix
)
from condor.config import (
    TERM_LIST_PATH,
    MATRIX_PATH,
    MODEL_PATH,
)


@click.group()
def model():
    """
    Model shortcut to handle both a matrix and a ranking
    at the same time.
    """
    pass


@model.command()    # noqa
@click.option('--target', default=None, type=str,
              help='Bibliography set to work with')
@click.option('--verbose/--quiet', default=False,
              help='Be more verbose')
def create(target, verbose):
    '''
    Creates a ranking matrix and a lsa model for the specified bibset.
    '''

    db = session()
    if target is None:
        bibset = db.query(BibliographySet).order_by(
            BibliographySet.created.desc()
        ).first()
    else:
        query = db.query(BibliographySet)
        query = query.filter(BibliographySet.eid.like(target + '%'))
        bibset = query.first()

    if bibset is None:
        click.echo('Please create a bibset first')
        sys.exit(1)

    click.echo('I will generate a model for the {} bibset...'.format(
        bibset.eid))

    words, frequency, options = build_matrix(bibset)
    nwords, nrecs = frequency.shape

    if verbose:
        click.echo('I\'ve built the frequency matrix with the folowing traits')
        click.echo('Number of records: {}'.format(nrecs))
        click.echo('Number of words: {}'.format(nwords))

    U, S, V = numpy.linalg.svd(frequency, full_matrices=False)
    ss = S / numpy.sum(S)
    ss = numpy.cumsum(ss)

    # Keep 80% of the covariance
    k = numpy.sum(ss < 0.8)

    acoted = numpy.dot(numpy.diag(S[:k]), V[:k, :])
    acoted = numpy.dot(U[:, :k], acoted)

    if verbose:
        nwords, nrecs = acoted.shape
        click.echo('I\'ve removed noise from the freq mat...')
        click.echo('Number of records: {}'.format(nrecs))
        click.echo('Number of words: {}'.format(nwords))

    matrix_hash = hashlib.sha1(
        '{}{}{}{}'.format(bibset.eid, bibset.modified, nrecs, nwords)
        .encode()
    ).hexdigest()
    matrix_filename = os.path.join(MATRIX_PATH, '{}.npy'.format(matrix_hash))
    click.echo(
        'Storing the term document matrix at {}'
        .format(matrix_filename)
    )
    numpy.save(matrix_filename, frequency)

    term_list_filename = os.path.join(
        TERM_LIST_PATH, '{}.txt'.format(matrix_hash)
    )
    click.echo(
        'Storing the term list at {}'.format(TERM_LIST_PATH)
    )
    with open(term_list_filename, 'w') as file:
        file.write('\n'.join(words))

    model_hash = hashlib.sha1(
        '{}{}{}{}condor'.format(bibset.eid, bibset.modified, nrecs, nwords)
        .encode()
    ).hexdigest()
    model_filename = os.path.join(MODEL_PATH, '{}.npy'.format(model_hash))
    click.echo(
        'Storing the ranking matrix at {}'
        .format(model_filename)
    )
    numpy.save(model_filename, acoted)
    td_matrix = TermDocumentMatrix(
        bibliography_options='',
        processing_options=options,
        term_list_path=term_list_filename,
        tdidf_matrix_path=matrix_filename,
    )
    td_matrix.bibliography_set = bibset
    td_matrix.ranking_matrices = [
        RankingMatrix(
            kind='lsa',
            build_options='',
            ranking_matrix_path=model_filename,
        )
    ]
    db.add(td_matrix)
    db.commit()



import pytest

from condor.record.util import isi_text_to_dic


@pytest.fixture(scope='module')
def isi_text(request):
    return '''
FN Thomson Reuters Web of Science™
VR 1.0
PT J
AU Jeong, K
   Hong, T
   Ban, C
   Koo, C
   Park, HS
AF Jeong, Kwangbok
   Hong, Taehoon
   Ban, Cheolwoo
   Koo, Choongwan
   Park, Hyo Seon
TI Life cycle economic and environmental assessment for establishing the
   optimal implementation strategy of rooftop photovoltaic system in
   military facility
SO JOURNAL OF CLEANER PRODUCTION
LA English
DT Article
DE Photovoltaic (PV) system; Life cycle cost; Life cycle CO2; Military
   facility; Gable roof
ID RENEWABLE ENERGY SYSTEM; EDUCATIONAL FACILITY; SOLAR; COST; ELECTRICITY;
   BUILDINGS; FRAMEWORK; MODEL; OPTIMIZATION; EMISSIONS
AB The Ministry of National Defense (South Korea) promotes its Defense Green \
    Growth policy to reduce greenhouse gas emissions. Based on this background\
, this study aimed to conduct the life cycle economic and environmental\
    assessment for establishing the optimal implementation strategy for \
    rooftop photovoltaic system in military facility. Considering three \
    factors (i.e., the orientation of the gable roof, the installation area of\
    the PV system, and the slope of the installed panel), 12 implementation \
    scenarios of PV system were established. The detailed results by prototype\
    are summarized in terms of the two perspectives (i.e., the absolute and \
    relative investment values): (i) Prototype 1 (south-north): P1-S/N (opt.) \
    in terms of the NPV25 (net present value at year 25) and P1-S (opt.) in \
    terms of the SIR25 (savings-to-investment ratio at year 25); (ii) \
    Prototype 2 (southeast-northwest): P2-SE/NW (opt.) in terms of the NPV25 \
    and P2-SE (ext.) in terms of the SIR25; and (iii) Prototype 3 (east-west):\
    P3-E/W (ext.) in terms of the NPV25 and P3-E (opt.) in terms of the SIR25.\
C1 [Jeong, Kwangbok; Hong, Taehoon; Ban, Cheolwoo; Koo, Choongwan; Park,
RP Hong, T (reprint author), Yonsei Univ, Dept Architectural Engn, Seoul
EM kbjeong7@yonsei.ac.kr; hong7@yonsei.ac.kr; qkscjfdn@naver.com;
   cwkoo@yonsei.ac.kr; hspark@yonsei.ac.kr
FU National Research Foundation of Korea (NRF) grant - Korea government
   (MSIP; Ministry of Science, ICT & Future Planning)
   [NRF-2012R1A2A1A01004376, 2011-0018360]
FX This work was supported by the National Research Foundation of Korea
   (NRF) grant funded by the Korea government (MSIP; Ministry of Science,
   ICT & Future Planning) (NRF-2012R1A2A1A01004376 & No. 2011-0018360).
CR Bravi M, 2014, J CLEAN PROD, V66, P301, DOI 10.1016/j.jclepro.2013.11.015
   CEC (California Energy Commission), 2001, GUID PHOT PV SYST DE
   CRS (Congressional Research Service), 2012, DEP DEF EN IN BACKGR
   Cucchiella F., 2015, J CLEAN PROD, V98, P241
   Dell'lsola A. J., 2003, LIFE CYCLE COSTING F
   Gupta MJ, 2002, ENERGY, V27, P777, DOI 10.1016/S0360-5442(02)00030-0
   Hong T, 2012, ENERG BUILDINGS, V45, P229, DOI 10.1016/j.enbuild.2011.11.006
   Hong T, 2014, RENEW SUST ENERG REV, V29, P286, DOI 10.1016/j.rser.2013.08.0\
61
   Hong T, 2014, ENERGY, V65, P190, DOI 10.1016/j.energy.2013.11.082
   Hong T, 2014, ENERG POLICY, V66, P157, DOI 10.1016/j.enpol.2013.10.057
   Hong T, 2013, APPL ENERG, V103, P539, DOI 10.1016/j.apenergy.2012.10.013
   Jeong K., 2014, J CLEAN PROD, V20, P1
   Ji C, 2014, ENERG BUILDINGS, V72, P186, DOI 10.1016/j.enbuild.2013.12.045
   KCER (Korean Certified Emission Reductions), 2014, KOR VOL EM RED
   KEI (Korea Environment Institute), 2012, INT TRENDS GHG RED P
   KEMCO (Korea Energy Management Corporation), 2014, AUT CALC TOE CO2 EM
   Kim C. J., 2015, ENV IMPACT ASSES, V54, P9
   Kim D, 2014, ENERGIES, V7, P5129, DOI 10.3390/en7085129
   KMKE (Korea Ministry of Knowledge Economy), 2012, REG GOV PROC TRAD ST
   Koo C, 2013, ENVIRON SCI TECHNOL, V47, P4829, DOI 10.1021/es303774a
   Koo C, 2014, PROG PHOTOVOLTAICS, V22, P462, DOI 10.1002/pip.2448
   KURC (Korea Urban Renaissance Center), 2010, 4 ANN RES DEV REP
   Lee M, 2014, ENVIRON SCI TECHNOL, V48, P4604, DOI 10.1021/es405293u
   MND (Ministry of National Defense), 2012, DEF MIL INST STAND
   MND (Ministry of National Defense), 2011, DEF GREEN GROWTH REF
   MND (Ministry of National Defense), 2012, DEF WHIT PAP
   MNR (Ministry of Natural Resources), 2004, CLEAN EN PROJ AN RET
   MNR (Ministry of Natural Resources), 2013, RETSCREEN 4
   Mohamad RS, 2014, J CLEAN PROD, V70, P78, DOI 10.1016/j.jclepro.2014.02.033
   NREL (National Renewable Energy Laboratory), 2011, LESS LEARN NET ZER E
   NREL (National Renewable Energy Laboratory), 2010, 2008 SOL TECHN MARK
   Othman NF, 2015, J CLEAN PROD, V91, P71, DOI 10.1016/j.jclepro.2014.12.044
   Swift KD, 2013, RENEW ENERG, V57, P137, DOI 10.1016/j.renene.2013.01.011
   Uddin MS, 2014, J CLEAN PROD, V69, P153, DOI 10.1016/j.jclepro.2014.01.073
   Urn T. J., 2012, P SAREK 2012 SUMM AN, V6, P301
NR 45
TC 0
Z9 0
PU ELSEVIER SCI LTD
PI OXFORD
PA THE BOULEVARD, LANGFORD LANE, KIDLINGTON, OXFORD OX5 1GB, OXON, ENGLAND
SN 0959-6526
EI 1879-1786
J9 J CLEAN PROD
JI J. Clean Prod.
PD OCT 1
PY 2015
VL 104
BP 315
EP 327
DI 10.1016/j.jclepro.2015.05.066
PG 13
WC Engineering, Environmental; Environmental Sciences
SC Engineering; Environmental Sciences & Ecology
GA CM3AG
UT WOS:000357552900031
ER
'''


def test_isi_record_to_dic(isi_text):
    dic = isi_text_to_dic(isi_text)
    assert ['J'] == dic['PT']
    assert 5 == len(dic['AU'])



"""
Implements the condor matrix commands to create term document matrices from
bibsets.
"""

import click


@click.group()
def matrix():
    """
    Term document matrix related commands.
    """
    pass


@matrix.command()
def create():
    """
    Create a new term document matrix.
    """
    pass


@matrix.command()
def list():
    """
    List all the available term document matrices.
    """
    pass


@matrix.command()
def delete():
    """
    Delete a given matrix and associated search engines.
    """
    pass



import pytest

from click.testing import CliRunner

from condor.scripts.bibset import create as bibcreate
from condor.scripts.model import create as modelcreate
from condor.scripts.query import query

from condor.scripts.cli import bibset
from condor.scripts.cli import matrix
from condor.scripts.cli import ranking


@pytest.fixture
def runner():
    return CliRunner()


def test_basic_usage(runner):
    res = runner.invoke(bibcreate, [])
    assert res.exit_code == 2
    assert 'Usage' in res.output


def test_populate_bibtex(runner):
    res = runner.invoke(
        bibcreate,
        ['--bib', '--verbose', 'data/bib/*.bib']
    )
    assert res.exit_code == 0
    assert 'The database contains 3 records' in res.output


def test_basic_usage_model(runner):
    res = runner.invoke(modelcreate, ['--help'])
    assert res.exit_code == 0
    assert 'Usage' in res.output


def test_basic_usagle_query(runner):
    res = runner.invoke(query, [])
    assert res.exit_code == 2
    assert 'Usage' in res.output


def test_condor_bibset_group(runner):
    res = runner.invoke(bibset, [])
    assert res.exit_code == 0
    assert 'Usage' in res.output


def test_condor_matrix_group(runner):
    res = runner.invoke(matrix, [])
    assert res.exit_code == 0
    assert 'Usage' in res.output


def test_condor_ranking_group(runner):
    res = runner.invoke(ranking, [])
    assert res.exit_code == 0
    assert 'Usage' in res.output



import bibtexparser
import hashlib
import re

from .base import RecordIterator
from .base import RecordParser
from .base import LanguageGuesser

from condor.normalize import LatexAccentRemover


class BibtexRecordParser(RecordParser):

    mappings = {
        'description': 'abstract',
    }

    guesser = LanguageGuesser()

    accent_remover = LatexAccentRemover()

    def _clear_keywords(self, raw):
        line = raw.get('keyword', '')
        return re.split(r'[,; ]+', line)

    def _clear_hash(self, raw):
        sha = hashlib.sha1()
        data = self.clear('title', raw) + self.clear('description', raw)
        sha.update(data.encode('utf-8'))
        return sha.hexdigest()

    def _clear_language(self, raw):
        data = self.clear('title', raw) + self.clear('description', raw)
        data += ' '.join(self.clear('keywords', raw))
        return raw.get('language', self.guesser.guess(data))

    def clear(self, field, raw):
        mapping = self.get_mapping(field)
        cleared = raw.get(mapping, super().clear(field, raw))
        if field in self.list_fields:
            return [self.accent_remover.apply_to(item) for item in cleared]
        return self.accent_remover.apply_to(cleared)

    def parse(self, raw):
        if isinstance(raw, str):
            return super().parse(bibtexparser.loads(raw).entries[0])
        return super().parse(raw)


class BibtexRecordIterator(RecordIterator):

    '''
    Iterates over bibtex reccords
    '''

    parser_class = BibtexRecordParser

    def get_buffer(self):
        with open(self.filename, 'r') as bibtex:
            database = bibtexparser.load(bibtex)
            for entry in database.entries:
                yield entry



'''
Implements the populate script.
'''

import glob
import itertools
import sys

import click
import sqlalchemy
import tabulate

from condor.record import BibtexRecordIterator
from condor.record import FroacRecordIterator
from condor.record import IsiRecordIterator

from condor.dbutil import session

from condor.models import Bibliography
from condor.models import BibliographySet


def recordset_class(name):
    '''
    Builds a record class out  of the `name` of the extension file.
    '''
    if name == 'isi':
        return IsiRecordIterator
    elif name == 'xml':
        return FroacRecordIterator
    elif name == 'froac':
        return FroacRecordIterator
    elif name == 'bib':
        return BibtexRecordIterator
    raise NotImplementedError('{} parser is not implemented yet'.format(name))


def describe_bibset(kind, pattern):
    desc = 'Bibliography set created from {kind} files ussing {pattern}'
    return BibliographySet(
        description=desc.format(kind=kind, pattern=pattern)
    )


def find_records(pattern, klass, verbose=False):
    for filename in glob.glob(pattern, recursive=True):
        if verbose:
            click.echo('I\'m processing file {}...'.format(filename))
        rs = klass(filename)
        for record in rs:
            yield record


def chunks(sequence, n):
    seq_it = iter(sequence)
    while True:
        chunk = itertools.islice(seq_it, n)
        try:
            first_el = next(chunk)
        except StopIteration:
            return
        yield itertools.chain((first_el, ), chunk)


def existing_bibliographies(db, bibset, hashes):
    query = db.query(Bibliography).join(BibliographySet)
    query = query.filter(BibliographySet.eid == bibset.eid)
    query = query.filter(Bibliography.hash.in_(hashes))
    return query.all()


@click.group()
def bibset():
    """
    Bibliography set related commands.
    """
    pass


@bibset.command()
@click.option('--count', default=10, help='Number of bibsets.')
def list(count):
    """
    List all the bibliography sets.
    """
    bibliography_sets = session().query(BibliographySet).order_by(
        BibliographySet.created.desc()
    ).limit(count)

    click.echo(
        tabulate.tabulate(
            [
                [
                    bibset.eid[:8],
                    bibset.description,
                    bibset.modified.strftime('%b %d, %Y, %I:%M%p'),
                    len(bibset.bibliographies)
                ]
                for bibset in bibliography_sets
            ],
            headers=[
                'Identifier', 'Description', 'Updated at', 'Docs count',
            ],
            tablefmt='rst',
        )
    )
    total = session().query(BibliographySet).count()
    if count >= total:
        click.echo('Showing all the bibsets.')
    else:
        click.echo(
            'Shwoing {count} out of {total} bibliography sets.'
            .format(count=count, total=total)
        )


@bibset.command()
@click.argument('target')
def delete(target):
    """
    Delete the target bibliography set.
    """
    db = session()
    try:
        bibliography_set = db.query(BibliographySet).filter(
            BibliographySet.eid.like(target + '%')
        ).one()
    except sqlalchemy.orm.exc.NoResultFound:
        click.echo('Could not find a result matching {}'.format(target))
        return
    except sqlalchemy.orm.exc.MultipleResultsFound:
        click.echo('Found many results matching {}'.format(target))
        return

    click.echo(
        'I will delete the bibliography set {}.'
        .format(bibliography_set.eid)
    )
    click.echo('And also {} bibliographies.'
               .format(len(bibliography_set.bibliographies)))
    click.echo('And also {} term document matrices.'
               .format(len(bibliography_set.term_document_matrices)))
    click.echo('And also {} search engines.'
               .format(len([
                   rm
                   for tdm in bibliography_set.term_document_matrices
                   for rm in tdm
               ])))
    click.confirm('Do you want me to delete all this information?', abort=True)
    db.delete(bibliography_set)
    db.commit()


@bibset.command()
@click.argument('pattern')
@click.option('--xml', 'kind', flag_value='xml', default=True,
              help='Use the xml parser (default)')
@click.option('--froac', 'kind', flag_value='froac', default=True,
              help='Use the xml froac parser (default)')
@click.option('--isi', 'kind', flag_value='isi',
              help='Use the isi plain text parser (default xml)')
@click.option('--bib', 'kind', flag_value='bib',
              help='Use the bibtex parser (default xml)')
@click.option('--verbose/--quiet', default=False,
              help='Be more verbose')
@click.option('--chunk-size', default=1000,
              help='Insert into db in chunks')
def create(pattern, kind, verbose, chunk_size):
    '''
    Populates a mongo database collection with all the records it can find
    in the files matching the provided `PATTERN`, the parser will be determined
    usint the kind flags.
    '''

    click.echo('I\'m looking for {} records in files matching {}'.format(
        kind, pattern))

    try:
        rs_class = recordset_class(kind)
    except NotImplementedError as e:
        click.echo('Sadly, ' + e.message)
        sys.exit(1)

    db = session()
    bibset = describe_bibset(kind, pattern)
    db.add(bibset)
    db.commit()

    click.echo('I\'m writting to {bibset.eid}'.format(bibset=bibset))

    records = find_records(pattern, rs_class, verbose=verbose)
    for chunk in chunks(records, chunk_size):
        record_hash = {
            record['hash']: record
            for record in chunk
        }

        # Filter out existing bibliography records
        for bib in existing_bibliographies(db, bibset, record_hash.keys()):
            record_hash.pop(bib.hash)

        db.bulk_insert_mappings(
            Bibliography,
            [
                dict(bibliography_set_eid=bibset.eid, **record)
                for record in record_hash.values()
            ]
        )
        db.commit()

    click.echo('And... I\'m done')
    click.echo('The database contains {} records'.format(
        db.query(Bibliography).join(BibliographySet).
        filter(BibliographySet.eid == bibset.eid).
        count()
    ))


if __name__ == "__main__":
    bibset()



'''
Contains utility functions to work with tokens and decorators
to work with XML, lists and generators.
'''

import collections
import functools
import re


def xml_to_text(func):
    '''
    Transforms a function that would return an XML element into a function
    that returns the text content of the XML element as a string.
    '''
    @functools.wraps(func)
    def inner(*args, **kwargs):
        result = func(*args, **kwargs)
        if result is None:
            return ''
        return result.firstChild.nodeValue
    return inner


def gen_to_list(func):
    '''
    Transforms a function that would return a generator into a function that
    returns a list of the generated values, ergo, do not use this decorator
    with infinite generators.
    '''
    @functools.wraps(func)
    def inner(*args, **kwargs):
        return list(func(*args, **kwargs))
    return inner


def isi_text_to_dic(text):
    '''
    This function takes in any ISI WOS plain text formatted string and turns it
    into a dictionary where the keys are the two letter leading keys and the
    values are a list of the strings under that key.
    '''
    fields = collections.defaultdict(list)
    curr = ''
    for line in re.split(r'\n+', text):
        name = line[:2]
        value = line[3:]
        if not name.isspace():
            curr = name
        if not curr.isspace():
            fields[curr].append(value)
    return fields


def to_list(obj):
    '''
    Transforms a non iterable object into a singleton list, or an iterable
    into a list.
    '''
    if isinstance(obj, list) or isinstance(obj, tuple):
        return list(obj)
    return [obj]



"""Helpers to build a term document matrix with several different options.

.. note:: this module can receive list of already resolved models but it should
    not use the database or anything related with sqlalchemy.
"""

import collections
import itertools
import numpy

from condor.normalize import CompleteNormalizer


BuildMatrixResult = collections.namedtuple(
    'BuildMatrixResult',
    ['words', 'matrix', 'options']
)


def build_matrix(bibset):
    """Builds a term document matrix using a bibliography set.
    """
    words = word_set(bibset)
    nwords = len(words)
    ndocs = len(bibset.bibliographies)
    frequency = numpy.zeros((ndocs, nwords), dtype=int)
    for row, col, freq in matrix(bibset, words):
        frequency[row, col] = freq
    return BuildMatrixResult(
        words,
        frequency,
        str(CompleteNormalizer.__mro__),
    )


def get_tokens(record, fields=None, list_fields=None):
    fields = fields or ['title', 'description', ]
    list_fields = list_fields or ['keywords', ]
    tokens = []
    for field in fields:
        tokens.extend(record.get(field, '').split())
    for field in list_fields:
        value = record.get(field, '{""}')
        if value == '{""}':
            continue
        tokens.extend(word for val in value[1:-1].split(',') for word in val.split())
    return tokens


def raw_data(bib):
    normalizer = CompleteNormalizer(language=bib.language)
    return [normalizer.apply_to(token) for token in get_tokens(bib.__dict__)]


def word_set(bibset):
    def words():
        for bib in bibset.bibliographies:
            yield raw_data(bib)
    return sorted(set(itertools.chain.from_iterable(words())))


def matrix(bibset, words):
    word_dict = {word: pos for pos, word in enumerate(words)}
    for ind, bib in enumerate(bibset.bibliographies):
        raw = list(raw_data(bib))
        frequency = collections.Counter(raw)
        for word, freq in frequency.items():
            yield ind, word_dict[word], freq



import hashlib
import uuid

from datetime import datetime

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import (
    Column,
    DateTime,
    ForeignKey,
    Unicode,
)


DeclarativeBase = declarative_base()


def eid_gen():
    sha = hashlib.sha1('{}'.format(uuid.uuid4()).encode())
    return sha.hexdigest()


class AuditableMixing(object):
    """
    Table mixing to add the common fields in our database, to make it auditable
    we add timestamps and an eid field as primary key just for convenience.
    """

    eid = Column(Unicode(40), primary_key=True, default=eid_gen)
    created = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow
    )
    modified = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow
    )



import click


from condor.scripts.bibset import bibset
from condor.scripts.model import model
from condor.scripts.query import query
from condor.scripts.matrix import matrix


@click.group()
def ranking():
    """
    Ranking matrix related commands.
    """
    pass


class CondorCommand(click.MultiCommand):

    COMMANDS = {
        'bibset': bibset,
        'model': model,
        'matrix': matrix,
        'ranking': ranking,
        'query': query,
    }

    def list_commands(self, ctx):
        return list(self.COMMANDS.keys())

    def get_command(self, ctx, name):
        return self.COMMANDS.get(name)


condor = CondorCommand(
    help='Condor information retrieval software'
)


if __name__ == "__main__":
    condor()



'''
Uses the latest model to query the database for the most important documents
according to lsa.
'''

import collections
import operator
import sys

import click
import numpy

from condor.normalize import CompleteNormalizer
from condor.models import RankingMatrix
from condor.dbutil import session


def frequency(words, tokens):
    # word_dict = {word: pos for pos, word in enumerate(words)}
    normalizer = CompleteNormalizer()
    frequency = collections.Counter(
        normalizer.apply_to(token)
        for token in tokens
    )
    return [frequency.get(word, 0) for word in words]


@click.command()
@click.argument('parameters', nargs=-1, required=True)
@click.option('--target', default=None, type=str)
@click.option('--verbose/--quiet', default=False,
              help='Be more verbose')
def query(parameters, target, verbose):
    '''
    Queries the database using the given parameters, the model that this
    script will pick up to do the query is the latest available model.
    '''

    click.echo('You queried: {}'.format(' '.join(parameters)))

    db = session()
    if target is None:
        ranking = db.query(RankingMatrix).order_by(
            RankingMatrix.created.desc()
        ).first()
    else:
        query = db.query(RankingMatrix)
        query = query.filter(RankingMatrix.eid.like(target + '%'))
        ranking = query.first()

    if ranking is None:
        click.echo('Please create a ranking first')
        sys.exit(1)

    click.echo('I will query the ranking for the {} bibset...'.format(
        ranking.eid))

    ranking_matrix = numpy.load(ranking.ranking_matrix_path)

    words_filename = ranking.term_document_matrix.term_list_path
    with open(words_filename, 'r') as words_file:
        words = words_file.read().split('\n')

    freq = frequency(words, parameters)
    documents = ranking.term_document_matrix.bibliography_set.bibliographies
    cos = numpy.dot(ranking_matrix, freq)
    if verbose:
        click.echo('Your words are: {}'.format(words))
        click.echo('Your frequency is: {}'.format(freq))
        click.echo('Your cosines are: {}'.format(cos.shape))
        click.echo('Got a model with {} shape'.format(ranking_matrix.shape))

    ordered = reversed(sorted(zip(documents, cos), key=operator.itemgetter(1)))
    ordered = list(ordered)
    for document, imp in ordered[:3]:
        click.echo('')
        click.echo('With an importance of {}:'.format(imp))
        click.echo(document.title)
        click.echo(document.description)



import pytest

from condor.record.base import LanguageGuesser


@pytest.fixture(scope='module')
def guesser(request):
    return LanguageGuesser()


def test_language_guesser_can_be_instantiated(guesser):
    assert guesser is not None


def test_language_guesser_counts_right(guesser):
    es_counts = guesser.counts('hola, ¿qué hace?')
    assert 3 == es_counts['es_CO']
    assert 0 == es_counts['en_US']
    assert 0 == es_counts['fr_FR']


def test_language_guesser_guesess_languages_for_different_sentences(guesser):
    assert 'spanish' == guesser.guess('hola, ¿qué hace?')
    assert 'english' == guesser.guess('hello, how are you?')
    assert 'french' == guesser.guess('je suis un homme')
    assert 'german' == guesser.guess('ich trinke das Wasser')


def test_language_guesser_falls_back_to_default_lang(guesser):
    assert 'english' == guesser.guess(
        'askdfjlask klajsd flkajslk ajlsdkfj alskdf'
    )


def test_language_guesser_checks_dominant_language(guesser):
    assert 'english' == guesser.guess('hello how are you man hola que hace')
    assert 'spanish' == guesser.guess('hello how are hola que hay de nuevo')


def test_language_guesser_preferences(guesser):
    assert 'english' == guesser.guess(
        'Communications in Computer and Information Science'
    )



from sqlalchemy import (
    Column,
    ForeignKey,
    Unicode,
)

from sqlalchemy.orm import relationship

from condor.models.base import (
    AuditableMixing,
    DeclarativeBase,
)


class Bibliography(AuditableMixing, DeclarativeBase):

    __tablename__ = 'bibliography'

    bibliography_set_eid = Column(
        Unicode(40),
        ForeignKey('bibliography_set.eid')
    )

    hash = Column(Unicode(40), nullable=False)
    title = Column(Unicode(512), nullable=False)
    description = Column(Unicode, nullable=False)
    keywords = Column(Unicode, nullable=False)
    language = Column(Unicode(16), nullable=False)
    full_text_path = Column(Unicode(512), nullable=True)

    bibliography_set = relationship(
        'BibliographySet',
        back_populates='bibliographies',
    )



'''
Utils to work with a mongo database, it contains a global connection to the
database so that a new one is not created with every request which is a huge
overhead. Furthermore it has a tool to use a pymongo collection as a context
manager.
'''

from sqlalchemy import create_engine
from sqlalchemy.orm import (
    scoped_session,
    sessionmaker,
)


def session():
    return scoped_session(sessionmaker(bind=create_engine(
        'postgresql://condor-ir:condor-ir@localhost/condor-ir'
    )))



'''
This module contains utilities to normalize words and simplify them, it should
be able to remove punctuation, filter out stopwords and maybe transform some
latex accents into unicode accent characters.
'''

import re
import string

from nltk.corpus import stopwords
from nltk.stem.snowball import SnowballStemmer


class SpaceTokenizer(object):

    '''
    Simple tokenization based on spaces
    '''

    def tokenize(self, text):
        return text.split()


class Normalizer(object):

    default_tokenizer = SpaceTokenizer
    default_language = 'spanish'

    def __init__(self, language=None, tokenizer=None):
        if language is not None:
            self.language = language.lower()
        else:
            self.language = self.default_language
        self.tokenizer = tokenizer or self.default_tokenizer()

    def apply_to(self, text):
        return text


class PunctuationRemover(Normalizer):

    '''
    Removes punctuation from a text
    '''

    characters = string.punctuation + '¡¿'

    def __init__(self, characters=None, **kwargs):
        self.translation = str.maketrans(
            dict.fromkeys(characters or self.characters)
        )
        super().__init__(**kwargs)

    def apply_to(self, text):
        return super().apply_to(text.translate(self.translation))


class Stemmer(Normalizer):

    '''
    Changes words to their respective stemms
    '''

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.stemmer = SnowballStemmer(self.language)

    def apply_to(self, text):
        tokens = self.tokenizer.tokenize(text)
        result = ' '.join(self.stemmer.stem(token) for token in tokens)
        return super().apply_to(result)


class StopwordRemover(Normalizer):

    '''
    Removes stopwords from a text
    '''

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.stopwords = {
            word: None for word in stopwords.words(fileids=self.language)
        }

    def apply_to(self, text):
        tokens = self.tokenizer.tokenize(text)
        result = ' '.join(
            token for token in tokens if token not in self.stopwords
        )
        return super().apply_to(result)


class Lowercaser(Normalizer):

    '''
    Changes case to lowercase through the normalizer API
    '''

    def apply_to(self, text):
        return super().apply_to(text.lower())


class LatexAccentRemover(Normalizer):

    '''
    Removes latex accents like `\\'{a}` and makes them unicode chars `á`.
    '''

    accents = {
        '\'': {
            'a': 'á',
            'e': 'é',
            'i': 'í',
            'o': 'ó',
            'u': 'ú',
        },
        '`': {
            'a': 'à',
            'e': 'è',
            'i': 'ì',
            'o': 'ò',
            'u': 'ù',
        },
        '~': {
            'n': 'ñ',
            'o': 'õ',
            'a': 'ã',
        },
        '^': {
            'a': 'â',
            'e': 'ê',
            'i': 'î',
            'o': 'ô',
            'u': 'û',
        },
        '"': {
            'a': 'ä',
            'e': 'ë',
            'i': 'ï',
            'o': 'ö',
            'u': 'ü',
            'y': 'ÿ',
        },
        'a': {
            'e': 'æ',
        },
        'c': {
            'c': 'ç',
        },
        'o': {
            'e': 'œ',
        },
        's': {
            's': 'ß',
        },
        'v': {
            's': 'š',
        },
    }

    formats = [
        r"{{\{accent}{{{character}}}}}",
        r"{{\{accent}{character}}}",
        r"\{accent}{{{character}}}",
        r"\{accent}{character}",
    ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _replacements(self):
        for accent, cases in self.accents.items():
            for character, modification in cases.items():
                for format_ in self.formats:
                    latex = format_.format(accent=accent, character=character)
                    yield (
                        latex,
                        modification,
                    )
                    yield (
                        latex.upper(),
                        modification.upper(),
                    )

    def apply_to(self, text):
        result = text
        for old, new in self._replacements():
            result = result.replace(old, new)
        return super().apply_to(result)


class CompleteNormalizer(LatexAccentRemover,
                         PunctuationRemover,
                         Lowercaser,
                         StopwordRemover,
                         Stemmer):

    '''
    A Normalizer that aggregates all the effects described in this module
    '''

    pass



'''
This module describes the API for a record parser and a record iterator, they
have the responsibility of transforming a chunk of text into a usable
dictionary and go through a file and find all the records.
'''

from enchant import request_dict
from collections import OrderedDict

from condor.normalize import PunctuationRemover
from condor.normalize import SpaceTokenizer


class RecordParser(object):

    '''
    Outlines the API for parsing different kinds of records into dictionaries
    that are easy to use and store.
    '''

    mappings = {}
    interest_fields = ['hash', 'title', 'keywords', 'description', 'language']
    list_fields = ['keywords', ]

    def get_mapping(self, field):
        '''
        Returns the name of a field with a modification useful when the field
        is easy to clear but it requires just a change of name.
        '''
        return self.mappings.get(field, field)

    def get_default(self, field):
        '''
        Returns the default value for a given `field`
        '''
        if field in self.list_fields:
            return []
        return ''

    def clear(self, field, raw):
        '''
        Clears the given `field` out of a raw data record.
        '''
        if hasattr(self, '_clear_' + field):
            return getattr(self, '_clear_' + field)(raw)
        return self.get_default(field)

    def parse(self, raw):
        '''
        Returns a dictionary of the interest fields in the metadata.
        '''
        data = {
            field: self.clear(field, raw)
            for field in self.interest_fields
        }
        return data


class RecordIterator(object):

    '''
    Iterates over a bunch of reccords in a file.
    '''

    parser_class = RecordParser

    def __init__(self, filename):
        self.filename = filename

    def get_buffer(self):
        raise NotImplementedError('Use an specialized class')

    def __iter__(self):
        '''
        Not implemented by default. Iterates over all the records in
        the file given by the filename.
        '''
        buff = self.get_buffer()
        parser = self.parser_class()
        for item in buff:
            yield parser.parse(item)


class LanguageGuesser(object):

    '''
    Guesses the language of a record if the record field is not
    defined
    '''

    languages = OrderedDict([
        ('en_US', 'english'),
        ('en_GB', 'english'),
        ('es_ES', 'spanish'),
        ('es_CO', 'spanish'),
        ('es_MX', 'spanish'),
        ('pt_BR', 'portuguese'),
        ('pt_PT', 'portuguese'),
        ('fr_FR', 'french'),
        ('fr_BE', 'french'),
        ('it_IT', 'italian'),
        ('de_DE', 'german'),
        ('de_CH', 'german'),
        ('de_AT', 'german'),
    ])

    default_lang = 'english'

    def __init__(self, languages=None):
        self.dictionaries = OrderedDict()
        for language in languages or self.languages:
            self.dictionaries[language] = request_dict(language)
        self.normalizer = PunctuationRemover()
        self.tokenizer = SpaceTokenizer()

    def counts(self, sentence):
        counts = OrderedDict()
        tokens = self.tokenizer.tokenize(
            self.normalizer.apply_to(sentence)
        )
        for lang, dictionary in self.dictionaries.items():
            counts[lang] = sum(dictionary.check(tk) for tk in tokens)
        return counts

    def guess(self, sentence):
        counts = self.counts(sentence)
        guessed_lang = None
        max_count = 0
        for lang, count in counts.items():
            if count > max_count:
                guessed_lang = lang
                max_count = count
        return self.languages.get(guessed_lang, self.default_lang)



import os

from setuptools import find_packages
from setuptools import setup


def get_install_requires():
    if os.environ.get('READTHEDOCS', None) == 'True':
        return [
            'mock>=1.3.0',
        ]
    return [
        'SQLAlchemy==1.0.12',
        'alembic==0.8.6',
        'bibtexparser>=0.6.2',
        'click>=6.2',
        'marshmallow>=2.4.2',
        'nltk>=3.1',
        'numpy>=1.9.2',
        'psycopg2==2.6.1',
        'pyenchant>=1.6',
        'tabulate>=0.7.5',
    ]


version = '1.0.0b2'

setup(
    name='condor-ir',
    version=version,
    author='Oscar David Arbeláez <@odarbelaeze>, German Augusto Osorio',
    author_email='odarbelaeze@gmail.com',
    packages=find_packages(),
    install_requires=get_install_requires(),
    tests_require=[
        'cov-core>=1.15.0',
        'coverage>=3.7.1',
        'py>=1.4.29',
        'pytest>=2.8.0',
        'pytest-cov>=1.8.1',
    ],
    setup_requires=[
        'pytest-runner>=2.7,<3dev',
    ],
    entry_points='''
    [console_scripts]
    condor=condor.scripts.cli:condor
    ''',
    url='https://condor-ir.co',
    download_url=(
        'https://github.com/odarbelaeze/condor-ir/tarball/{}'
        .format(version)
    ),
    keywords=['lsa', 'search', 'search engine', 'semantics', ],
    description='A latent semantic search engine implementation',
)



from .base import RecordIterator
from .base import RecordParser

from .util import isi_text_to_dic


class IsiRecordParser(RecordParser):

    '''This represents an ISI web of knowledge record'''

    mappings = {
        'title': 'TI',
        'description': 'AB',
        'keywords': 'ID',
        'language': 'LA',
        'hash': 'UT',
    }

    def parse(self, raw):
        '''
        Checks if the input is a string if so it transforms it into a
        dictionary and runs the default `parse` method.
        '''
        if isinstance(raw, str):
            return super().parse(isi_text_to_dic(raw))
        return super().parse(raw)

    def clear(self, field, raw):
        '''
        Uses the `mapings` to get the entries out of the raw dictionary
        '''
        _default = self.get_default(field)
        _result = super().clear(field, raw)
        if _result != _default:
            return _result
        _field = self.get_mapping(field)
        if field in self.list_fields:
            return raw.get(_field, _default)
        return ' '.join(raw.get(_field, []))


class IsiRecordIterator(RecordIterator):

    '''
    Iterates over a file with ISI txt reccords while yielding reccords.
    '''

    parser_class = IsiRecordParser

    def get_buffer(self):
        '''
        Iterates over a file by looking for lines containing the ER mark
        of the isi plain text files.
        '''
        buff = []
        for line in open(self.filename):
            buff.append(line)
            if line[:2] == 'ER':
                yield '\n'.join(buff)
                buff = []



from sqlalchemy import (
    Column,
    ForeignKey,
    Unicode,
)

from sqlalchemy.orm import relationship

from condor.models.base import (
    AuditableMixing,
    DeclarativeBase,
)


class TermDocumentMatrix(AuditableMixing, DeclarativeBase):

    __tablename__ = 'term_document_matrix'

    bibliography_set_eid = Column(
        Unicode(40),
        ForeignKey('bibliography_set.eid')
    )

    bibliography_options = Column(Unicode(512), nullable=False)
    processing_options = Column(Unicode(512), nullable=False)
    term_list_path = Column(Unicode(512), nullable=False)
    tdidf_matrix_path = Column(Unicode(512), nullable=False)

    bibliography_set = relationship(
        'BibliographySet',
        back_populates='term_document_matrices',
    )

    ranking_matrices = relationship(
        'RankingMatrix',
        back_populates='term_document_matrix',
        cascade='all, delete-orphan',
    )



import hashlib

from xml.dom import minidom

from .base import RecordIterator
from .base import RecordParser

from .util import gen_to_list
from .util import xml_to_text


class FroacRecordParser(RecordParser):

    language_key = {
        'es': 'spanish',
        'en': 'english',
        'pt': 'portuguese',
        'fr': 'french',
        'it': 'italian',
        'de': 'german',
    }

    default_language = 'english'

    def parse(self, raw):
        if isinstance(raw, str):
            return super().parse(minidom.parseString(raw))
        return super().parse(raw)

    @xml_to_text
    def _clear_title(self, raw):
        return raw.getElementsByTagName('lom:title').item(0)

    @xml_to_text
    def _clear_description(self, raw):
        return raw.getElementsByTagName('lom:description').item(0)

    def _clear_language(self, raw):
        langNode = raw.getElementsByTagName('lom:language').item(0)
        if langNode is None:
            return self.default_language
        lang = langNode.firstChild.nodeValue
        return self.language_key.get(lang, self.default_language)

    @gen_to_list
    def _clear_keywords(self, raw):
        keywords = raw.getElementsByTagName('lom:keyword')
        for keyword in keywords:
            yield keyword.firstChild.nodeValue

    def _clear_hash(self, raw):
        sha = hashlib.sha1()
        sha.update(raw.toxml().encode('utf-8'))
        return sha.hexdigest()


class FroacRecordIterator(RecordIterator):

    '''
    Iterates plain txt froac records in a file.
    '''

    parser_class = FroacRecordParser

    def get_buffer(self):
        dom = minidom.parse(self.filename)
        for dom_element in dom.getElementsByTagName('record'):
            yield dom_element



import pytest

from condor.dbutil import session as Session
from condor.models import (
    Bibliography,
    BibliographySet,
    TermDocumentMatrix,
    RankingMatrix,
)


@pytest.yield_fixture(scope='function')
def session():
    _session = Session()
    yield _session
    _session.rollback()
    _session.close()


@pytest.fixture
def bibset(session):
    bibset = BibliographySet(description='asdkfjaskldf')
    session.add(bibset)
    session.flush()
    return bibset


def test_bibliographies_are_deleted(session, bibset):
    bibliography = Bibliography(bibliography_set_eid=bibset.eid)
    session.flush()
    bib_eid = bibliography.eid
    session.delete(bibset)
    session.flush()
    assert session.query(Bibliography).filter(
        Bibliography.eid == bib_eid
    ).first() is None


def test_matrices_and_engines_are_deleted(session, bibset):
    term_matrix = TermDocumentMatrix(
        bibliography_set_eid=bibset.eid,
        bibliography_options='',
        processing_options='',
        term_list_path='',
        tdidf_matrix_path='',
    )
    session.add(term_matrix)
    session.flush()
    term_eid = term_matrix.eid
    rank_matrix = RankingMatrix(
        term_document_matrix_eid=term_eid,
        kind='',
        build_options='',
        ranking_matrix_path='',
    )
    session.add(rank_matrix)
    session.flush()
    rank_eid = rank_matrix.eid
    assert term_eid is not None
    assert rank_eid is not None
    session.delete(bibset)
    session.flush()
    assert session.query(TermDocumentMatrix).filter(
        TermDocumentMatrix.eid == term_eid
    ).first() is None
    assert session.query(RankingMatrix).filter(
        RankingMatrix.eid == rank_eid
    ).first() is None


def test_engines_are_deleted(session, bibset):
    term_matrix = TermDocumentMatrix(
        bibliography_set_eid=bibset.eid,
        bibliography_options='',
        processing_options='',
        term_list_path='',
        tdidf_matrix_path='',
    )
    session.add(term_matrix)
    session.flush()
    term_eid = term_matrix.eid
    rank_matrix = RankingMatrix(
        term_document_matrix_eid=term_eid,
        kind='',
        build_options='',
        ranking_matrix_path='',
    )
    session.add(rank_matrix)
    session.flush()
    rank_eid = rank_matrix.eid
    assert term_eid is not None
    assert rank_eid is not None
    session.delete(bibset)
    session.flush()
    assert session.query(TermDocumentMatrix).filter(
        TermDocumentMatrix.eid == term_eid
    ).first() is None
    assert session.query(RankingMatrix).filter(
        RankingMatrix.eid == rank_eid
    ).first() is None
